@@ -1,29 +1,132 @@
-import zlib, struct, time, os
+import zlib, struct, time, os, io
 import datetime as dt
 import pandas as pd
 import numpy as np
 import xml.etree.ElementTree as ET
 
+class Properties:
+    def Unserialize(bytesStream):
+        
+        properties = dict()
+        
+        version = bytesStream.read(1)[0]
+            
+        if version != 0:
+            print ("Error unserializing properties, version " + str.from_bytes(version) + " is not supported")
+            
+        itemsCount = bytesStream.read(4)
+        intItemsCount = int.from_bytes(itemsCount, "little")
+        
+        for items in range(intItemsCount): 
+            
+            strLen = bytesStream.read(1)[0]
+            
+            key = bytesStream.read(strLen).decode()
+            
+            dataTypeEnum = bytesStream.read(1)[0]
+            
+            dataType, size = _getDatatype(dataTypeEnum)
+            if dataType == "s":#string
+                value = _readString(bytesStream)
+            elif dataType == "D":#Datetime
+                value = _readString(bytesStream)
+                value = value.rstrip('0') #Remove trailing 0s
+                value = dt.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
+            else:    
+                value = struct.unpack(dataType, bytesStream.read(size))[0]
+            
+            properties[key] = value
+            
+        if 'Increment' in properties:
+            properties['Increment'] = dt.timedelta(milliseconds = properties['Increment'] )
+                
+        return properties
+
+class Binary:
+    def __init__(self):
+        self._header = FrameHeader("BLOB")
+        self._buffer = []
+        self._properties = dict()
+    
+    def Unserialize(self,bytesStream):
+        #Properties  
+        self._properties = Properties.Unserialize(bytesStream)
+        #Inner
+        innerLength = struct.unpack('i', bytesStream.read(4))[0]            
+        self._buffer = bytesStream.read(innerLength)
+
+class Composite:
+    def __init__(self):
+        self._header = FrameHeader("CPST")
+        self._frames = []
+        self._hash = dict()
+        self._properties = dict()
+        
+    def Unserialize(self,bytesStream):
+        #Properties  
+        self._properties = Properties.Unserialize(bytesStream)
+        #Inner
+        count = struct.unpack("h", bytesStream.read(2))[0]         
+        for element in range(count):
+            
+            bag = Bag()
+            
+            contains = struct.unpack("?", bytesStream.read(1))[0]
+            if contains:
+                strLen = bytesStream.read(1)[0]
+                bag.Key= bytesStream.read(strLen).decode()
+                self._hash[bag.Key] = bag
+                
+            frameHeader = FrameHeader(None)
+            frameHeader._tag = bytesStream.read(4)
+            frameHeader._version = bytesStream.read(1)[0]
+            
+            if frameHeader._version == b'\xFF':
+                print("Corrupt header, invalid version.")
+                
+            frameHeader._flags = bytesStream.read(1)[0] & 0xF0 #Only read what I understand
+            frameHeader._rawSize = struct.unpack("i", bytesStream.read(4))[0]
+            
+            if frameHeader._rawSize == 0:
+                print("Corrupt header, tag size can't be zero.")
+                
+            frameBase = FrameBase(frameHeader._tag) 
+            frameBase._header = frameHeader
+            frameBase._baseframe.Unserialize(bytesStream)
+      
+            bag._frame = frameBase
+            self._frames.append(bag)   
+        
 class Bag:
-    Key = str()
-    _frame = None
+    def __init__(self):
+        self.Key = str()
+        self._frame = None
 
 class FrameHeader:
-    _version = 0;
-    _flags = 0;
-    _rawSize = 0;
-    _tag = None; 
+    def __init__(self,tag):
+        self._version = 0;
+        self._flags = 0;
+        self._rawSize = 0;
+        self._tag = tag; 
     
 class FrameBase:
-    _header = FrameHeader()
+    def __init__(self,tagName):
+        self._header = FrameHeader(tagName)
+        if tagName == b'BLOB':
+            self._baseframe = Binary()
+        elif tagName == b'CPST':
+            self._baseframe = Composite()
+        else:
+            raise Exception("Invalid tag", tagName, "for FrameBase")
 
 class LogData:
-    _fileName = str()
-    _data = None
-    _mainTrendsNames = None
-    _mainDivisors = None
-    dataFrame = pd.DataFrame()
-    properties = None
+    def __init__(self):
+        self._fileName = str()
+        self._data = None
+        self._mainTrendsNames = None
+        self._mainDivisors = None
+        self.dataFrame = pd.DataFrame()
+        self.properties = None
 
 def _getDatatype (dataTypeEnum):
     thisdict = {
@@ -95,25 +198,22 @@ def _getDivisors(divList, RegModeList):
             divColumnReturn.append(div)
     return divColumnReturn
 
-def _readString(bytesArray,bytePos):
+def _readString(bytesStream):
     #https://docs.microsoft.com/en-us/openspecs/sharepoint_protocols/ms-spptc/1eeaf7cc-f60b-4144-aa12-4eb9f6e748d1?redirectedfrom=MSDN
-    pos = bytePos
     index = 0
     strLen = 0
     while True:
-        length = bytesArray[pos]
+        length = bytesStream.read(1)[0]
         strLen |= (length & 0x7F) << (7*index)
         index += 1
-        pos += 1
+
         if length & 0x80 == 0: #Number complete
             break
                
-    value = bytesArray[pos:pos+strLen].decode()
-    pos += strLen
-    
-    return value, pos
+    value = bytesStream.read(strLen).decode()  
+    return value
 
-def _processVersionN(version, Properties, bytesArray, innerBufferSize):
+def _processVersionN(version, composite):
     """
     Unpacks binary file according to its properties
 
@@ -122,30 +222,23 @@ def _processVersionN(version, Properties, bytesArray, innerBufferSize):
     version : int
         binary version  
         
-    Properties : dict
-        Dictionary containing at least 'StartTime', 'Increment' & 'Definition' properties
+    composite : Composite
+        Composite containing properties and frames
     
-    bytesArray : byte[] 
-        array containing the file to process  
-        
-    innerBufferSize : int
-        size of samples data
-        
     Returns
     -------
     LogData
         Structure containing most file data
 
     """
-    # utcStartTime = Properties['StartTime']
-    # incrementMillisecs = Properties['Increment']
+    # utcStartTime = composite._properties['StartTime']
+    # incrementMillisecs = composite._properties['Increment']
     # incrementTicks = incrementMillisecs * 10000.0
-    # definition = Properties['Definition']
+    # definition = composite._properties['Definition']
     
+    root = ET.fromstring(composite._properties["Definition"])
     
-    root = ET.fromstring(Properties["Definition"])
-    
-    if version == 3:
+    if version == 3: ##TODO: Use full definition instead of definition
         nodes = root.findall(".//object[@logged='y']/field")
     elif version == 2:
         nodes = root.findall(".//object[@logged='y']/field")
@@ -201,7 +294,11 @@ def _processVersionN(version, Properties, bytesArray, innerBufferSize):
     
     #Obtain stream data from InnerBuffer
     appendListOfSamples = listOfSamples.append
-    dataArray = np.ndarray(shape=(int((innerBufferSize/2)/len(fieldTypes)),len(fieldTypes)), dtype='h', buffer=bytesArray)
+    
+    bytesArray = composite._hash["RawData"]._frame._baseframe._buffer
+    bufferSize = len(composite._hash["RawData"]._frame._baseframe._buffer)
+    
+    dataArray = np.ndarray(shape=(int((bufferSize/2)/len(fieldTypes)),len(fieldTypes)), dtype='h', buffer=bytesArray)
     
     bitMapCount = 0    
     for i in range(len(dataArray[0,:])):
@@ -220,7 +317,7 @@ def _processVersionN(version, Properties, bytesArray, innerBufferSize):
     logData._data = arraySamples
     logData._mainTrendsNames = trendsNames
     logData._mainDivisors = divisors
-    logData.properties = Properties
+    logData.properties = composite._properties
     
     return logData
 
@@ -239,7 +336,7 @@ def binStreamToDF(file):
     LogData
         Structure containing most file data
 
-    """
+    """   
     #header
     TAGS = file.read(4)
     if TAGS != b'TAGS':
@@ -253,115 +350,24 @@ def binStreamToDF(file):
     else:
         compressed = False
     
-    #properties    
+    composite = Composite()
+      
     bytePos = 0    
     if compressed:
         bytesArray = zlib.decompress(file.read())
     else:
         bytesArray = file.read()
-        
-    version = bytesArray[bytePos]
+    bytesStream = io.BytesIO(bytesArray)
     
-    if version != 0:
-        print ("Error unserializing properties, version " + version + " is not supported")
+    composite.Unserialize(bytesStream)
         
-    bytePos+=1
-    itemsCount = bytesArray[bytePos:bytePos+4]
-    intItemsCount = int.from_bytes(itemsCount, "little")
-    bytePos+=4
-    
-    Properties = dict()
-    
-    for items in range(intItemsCount): 
-        
-        strLen = bytesArray[bytePos]
-        bytePos += 1
-        
-        key = bytesArray[bytePos:bytePos+strLen].decode()
-        bytePos += strLen
-        
-        dataTypeEnum = bytesArray[bytePos]
-        bytePos += 1
-        
-        dataType, size = _getDatatype(dataTypeEnum)
-        if dataType == "s":#string
-            value, bytePos = _readString(bytesArray, bytePos)
-        elif dataType == "D":#Datetime
-            value, bytePos = _readString(bytesArray, bytePos)
-            value = value.rstrip('0') #Remove trailing 0s
-            value = dt.datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
-        else:    
-            value = struct.unpack(dataType, bytesArray[bytePos:bytePos+size])[0]
-            bytePos += size
-    
-        Properties[key] = value
-    
-    if 'Increment' in Properties:
-        Properties['Increment'] = dt.timedelta(milliseconds = Properties['Increment'] )
-    
-    count = struct.unpack("h", bytesArray[bytePos:bytePos+2])[0]
-    bytePos += 2
-    
-    _hash = dict()
-    _frames = []
-    
-    for element in range(count):
-        
-        bag = Bag()
-        
-        contains = struct.unpack("?", bytesArray[bytePos:bytePos+1])[0]
-        bytePos += 1
-        
-        if contains:
-            strLen = bytesArray[bytePos]
-            bytePos += 1
-            bag.Key= bytesArray[bytePos:bytePos+strLen].decode()
-            bytePos += strLen
-            _hash[bag.Key] = bag
-            
-        frameHeader = FrameHeader()
-        frameHeader._tag = bytesArray[bytePos:bytePos+4]
-        bytePos += 4
-        frameHeader._version = bytesArray[bytePos]
-        bytePos += 1
-        if frameHeader._version == b'\xFF':
-            print("Corrupt header, invalid version.")
-        frameHeader._flags = bytesArray[bytePos] & 0xF0 #Only read what I understand
-        bytePos += 1
-        frameHeader._rawSize = struct.unpack("i", bytesArray[bytePos:bytePos+4])[0]
-        bytePos += 4
-        if frameHeader._rawSize == 0:
-            print("Corrupt header, tag size can't be zero.")
-            
-        frameBase = FrameBase()
-        frameBase._header = frameHeader
-        
-        if frameBase._header._tag != b"BLOB":
-            print("Just BLOB validated for now")
-            
-        #Inner Properties 
-        innerVersion = bytesArray[bytePos]
-        bytePos += 1    
-        if innerVersion != 0:
-            print ("Error unserializing properties, version " + innerVersion + " is not supported")
-            
-        innerItemsCount = struct.unpack('i', bytesArray[bytePos:bytePos+4])[0]
-        bytePos+=4
-        if innerItemsCount!=0: #itemsCount should be zero
-            print("Error")
-        innerLength = struct.unpack('i', bytesArray[bytePos:bytePos+4])[0]
-        bytePos+=4
-    
-        bag._frame = frameBase
-        _frames.append(bag)   
-        
-    if 'Version' in Properties:
-        if Properties['Version'] == 1:
-            logData = _processVersionN(1, Properties, bytesArray[bytePos:bytePos+innerLength], innerLength)
-        elif Properties['Version'] == 2:
-            logData = _processVersionN(2, Properties, bytesArray[bytePos:bytePos+innerLength], innerLength)
-        elif Properties['Version'] == 3:
-            logData = _processVersionN(3, Properties, bytesArray[bytePos:bytePos+innerLength], innerLength)
+    if 'Version' in composite._properties:
+        if composite._properties['Version'] == 1:
+            logData = _processVersionN(1, composite)
+        elif composite._properties['Version'] == 2:
+            logData = _processVersionN(2, composite)
+        elif composite._properties['Version'] == 3:
+            logData = _processVersionN(3, composite)
         else:
             print("Unknown version")
     
@@ -369,9 +375,7 @@ def binStreamToDF(file):
     
     step = np.arange(start = 0, stop=len(logData._data), step=1, dtype=np.int16)
     listOfTimeStamps = logData.properties['StartTime']+logData.properties['Increment']*pd.Series(step)
-    dFrame["Timestamp"] = listOfTimeStamps
-    dFrame.set_index("Timestamp",inplace=True)
-        
+    
     try:
         divColumns = _getDivisors(logData._mainDivisors,dFrame["RegStatus.RegulationMode"] )
     except:
@@ -380,6 +384,9 @@ def binStreamToDF(file):
     logData.dataFrame = dFrame.div(divColumns, axis='columns')
     del logData._data
     
+    logData.dataFrame.insert(0,"Timestamp", listOfTimeStamps)
+    # logData.dataFrame.set_index("Timestamp",inplace=True)
+        
     return logData
 
 def binFileToDF(path):
@@ -404,4 +411,4 @@ def binFileToDF(path):
     logData._fileName = os.path.basename(path)
     
     file.close()
-    return logData
\ No newline at end of file
+    return logData
